﻿using ProjectExcavator.DrawingObjects;
using ProjectExcavator.Generics;
using ProjectExcavator.MovementStrategy;

namespace ProjectExcavator
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawningExcavator
    /// </summary>
    public partial class FormExcavatorCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly ExcavatorGenericCollection<DrawingExcavator,
        DrawingObjectExcavator> _excavator;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormExcavatorCollection()
        {
            InitializeComponent();
            _excavator = new ExcavatorGenericCollection<DrawingExcavator,
            DrawingObjectExcavator>(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddEx_Click(object sender, EventArgs e)
        {
            FormExcavator form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                int result = _excavator + form.SelectedExcavator;
                if (result != -2)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = _excavator.ShowExcavator();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveEx_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (_excavator - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = _excavator.ShowExcavator();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _excavator.ShowExcavator();
        }

        private void buttonGeneration_Click(object sender, EventArgs e)
        {
            RandGeneration form = new();
            form.ShowDialog();
        }
    }
}
